(function() {
  var LazyLoader, Module, createDirProperty, createFileProperty, createLoader, exports, fs, parseName, pathUtil;

  Module = require('module');

  pathUtil = require('path');

  fs = require('fs');

  parseName = function(folder, file) {
    var extname, fullpath, name;
    extname = pathUtil.extname(file);
    name = pathUtil.basename(file, extname);
    fullpath = pathUtil.join(folder, file);
    return {
      fullpath: fullpath,
      name: name,
      extname: extname
    };
  };

  createFileProperty = function(instance, name, path) {
    return Object.defineProperty(instance, name, {
      get: function() {
        return require(path);
      }
    });
  };

  createDirProperty = function(instance, name, path) {
    return Object.defineProperty(instance, name, {
      get: function() {
        return new LazyLoader(path);
      }
    });
  };

  LazyLoader = (function() {
    function LazyLoader(__rootPath) {
      var extname, file, files, fullpath, name, stat, _i, _len, _ref;
      this.__rootPath = __rootPath;
      files = fs.readdirSync(this.__rootPath);
      for (_i = 0, _len = files.length; _i < _len; _i++) {
        file = files[_i];
        _ref = parseName(this.__rootPath, file), fullpath = _ref.fullpath, name = _ref.name, extname = _ref.extname;
        stat = fs.lstatSync(fullpath);
        if (stat.isFile()) {
          if (Module._extensions[extname] != null) {
            createFileProperty(this, name, fullpath);
          }
        } else if (stat.isDirectory()) {
          createDirProperty(this, name, fullpath);
        }
      }
    }

    return LazyLoader;

  })();

  createLoader = LazyLoader.create = function(rootPath) {
    return new LazyLoader(rootPath);
  };

  exports = module.exports = createLoader;

  exports.LazyLoader = LazyLoader;

}).call(this);
